SPACING = 1000 / 60;	// Build for 60 LEDs per meter
INCHES_TO_MM = 25.4;
HEIGHT = INCHES_TO_MM * 4.0;
BULBOUS_POWER = 0.5;	// weight the curve
NOZZLE_SIZE = 0.35;	// Replicator 2 == 0.35mm
CLIP = 0.01;
$fn = 120;

HOLLOW_WALL = NOZZLE_SIZE * 3;
CORE = NOZZLE_SIZE * 6;

// Hmm. How to create a bulbous spire shape?
// I could extrude two 2D fins, and perform an intersection().
function fin_radius_at_step( atStep, maxSteps ) = pow( cos( 90 * (atStep/maxSteps) ), BULBOUS_POWER );

function fin_points_left( ar, width, height, atStep, maxSteps ) =
	(atStep > maxSteps) ? ar :
		fin_points_left( concat( ar, [ 
			[ 
				-(width/2) * fin_radius_at_step(atStep,maxSteps),
				height * (atStep/maxSteps) 
			]
		] ), width, height, atStep+1, maxSteps );
		
function fin_points_right( ar, width, height, atStep, maxSteps ) =
	(atStep > maxSteps) ? ar :
		fin_points_right( concat( [
			[
				(width/2) * fin_radius_at_step(atStep,maxSteps),
				height * (atStep/maxSteps) 
			]
		], ar ), width, height, atStep+1, maxSteps );

module Fin2D( width, height, maxSteps, inset ) {
	left = fin_points_left( [], width, height, 0, maxSteps );
	right = fin_points_right( [], width, height, 0, maxSteps );
	pts = concat( left, right );
	intersection(){
		translate( [inset,0] ) polygon( points=pts );
		translate( [-inset,0] ) polygon( points=pts );
	}
}

module Fin3D( width, height, maxSteps, inset ) {
	rotate([90,0,0]) {
		intersection(){
			linear_extrude( height=width, center=true ) Fin2D( SPACING, HEIGHT, $fn, inset );
			rotate([0,90,0]) linear_extrude( height=width, center=true ) Fin2D( SPACING, HEIGHT, $fn, inset );
		}
	}
}

// Hmm. Draw an 'X' or something?
STRUT_HEIGHT = 1.5 * INCHES_TO_MM;
STRUT_RADIUS = NOZZLE_SIZE * 1.5;
module CoreStruts( width ) {
	for( i=[0:90:360] ){
		rotate([0,0,i]) {
			hull(){
				cylinder( h=STRUT_HEIGHT, r=STRUT_RADIUS, $fn=8 );
				translate([width/2,width/2,width*2])
					cylinder( h=STRUT_HEIGHT, r=STRUT_RADIUS, $fn=8 );
			}
		}
	}
}

BASE = SPACING + HOLLOW_WALL;

// Solid
translate([0,0,0]) {
	Fin3D( BASE, HEIGHT, $fn, 0 );	// solid
}

// Hollow
translate([SPACING,0,0]){
	difference(){
		// solid exterior
		Fin3D( BASE, HEIGHT, $fn, 0 );	
		// hollow interior
		translate([0,0,-CLIP]) Fin3D( BASE, HEIGHT, $fn, HOLLOW_WALL );	
	}
}

// Hollow, thick wall
translate([SPACING*2,0,0]){
	difference(){
		// solid exterior
		Fin3D( BASE, HEIGHT, $fn, 0 );	
		// hollow interior
		translate([0,0,-CLIP]) Fin3D( BASE, HEIGHT, $fn, HOLLOW_WALL*3 );	
	}
}

// Solid Core
translate([SPACING*3,0,0]){
	difference(){
		// solid exterior
		Fin3D( BASE, HEIGHT, $fn, 0 );	
		// hollow interior
		translate([0,0,-CLIP]) Fin3D( BASE, HEIGHT, $fn, HOLLOW_WALL );	
	}
	// Solid core
	Fin3D( BASE, HEIGHT, $fn, HOLLOW_WALL+CORE );
	// Struts
	/*
	intersection(){
		Fin3D( BASE, HEIGHT, $fn, 0 );	
		CoreStruts( BASE );
	}
	*/
	// Side supports
	translate([0,0,HOLLOW_WALL/2])
		cube( [BASE - 2*(HOLLOW_WALL+CORE), BASE-HOLLOW_WALL, HOLLOW_WALL], center=true );
}

// Hollow core
translate([SPACING*4,0,0]){
	difference(){
		// solid exterior
		Fin3D( BASE, HEIGHT, $fn, 0 );	
		// hollow interior
		translate([0,0,-CLIP]) Fin3D( BASE, HEIGHT, $fn, HOLLOW_WALL*0.667 );	
	}
	difference(){
		// Solid core
		union(){
			Fin3D( BASE, HEIGHT, $fn, HOLLOW_WALL+CORE );
				translate([0,0,HOLLOW_WALL/2])
		cube( [BASE - 2*(HOLLOW_WALL+CORE), BASE-HOLLOW_WALL, HOLLOW_WALL], center=true );
		}
		// Hollow core
		translate([0,0,-CLIP]) Fin3D( BASE, HEIGHT, $fn, HOLLOW_WALL*2*0.667+CORE );	
	}
}


// Light pipes
function wrap( n, height ) = n - n*(floor(n/height));

function weight_low( n ) = pow( wrap(n,180)/180, 20.0 );

PIPE_RADIUS = NOZZLE_SIZE * 2;
translate([SPACING*5,0,0]){
	difference(){
		// solid exterior
		Fin3D( BASE, HEIGHT, $fn, 0 );	
		// hollow interior
		translate([0,0,-CLIP]) Fin3D( BASE, HEIGHT, $fn, HOLLOW_WALL );	
	}
	intersection(){
		// solid exterior
		Fin3D( BASE, HEIGHT, $fn, 0 );
		union(){
			for( i=[0:1.5:180] ) {
				rotate( [weight_low(i),weight_low(i*3.579),weight_low(i*11.2967)] )
					cylinder( $fn=8, center=true, r=PIPE_RADIUS, h=HEIGHT*2 );
			}
		}
	}
}

SEED = 27;
randNums = rands(-1,1,360*3,SEED);	// X, Y, rotation

// Snarge
translate([SPACING*6,0,0]){
	difference(){
		// solid exterior
		Fin3D( BASE, HEIGHT, $fn, 0 );	
		// hollow interior
		translate([0,0,-CLIP]) Fin3D( BASE, HEIGHT, $fn, HOLLOW_WALL*0.667 );	
	}
	intersection(){
		// solid exterior
		Fin3D( BASE, HEIGHT, $fn, 0 );
		union(){
			for( i=[0:1:105] ) {
				idx = i * 3;
				translate( [randNums[idx]*(BASE/2), randNums[idx+1]*(BASE/2), BASE+i*0.9] )
					rotate( [40,0,randNums[idx+2]*180] )
						cylinder( $fn=8, center=true, r=NOZZLE_SIZE, h=HEIGHT*2 );
			}
		}
	}
}

// Hollow, with thick walls at base, thin at top
translate([SPACING*7,0,0]){
	difference(){
		// solid exterior
		Fin3D( BASE, HEIGHT, $fn, 0 );	
		// hollow interior
		translate([0,0,-CLIP]) Fin3D( BASE-HOLLOW_WALL*8, HEIGHT, $fn, NOZZLE_SIZE );	
	}
}


// Supporting walls

translate([-SPACING/2,(SPACING/2)-HOLLOW_WALL,0]){
	cube([8*SPACING,HOLLOW_WALL,0.5*INCHES_TO_MM]);
}
translate([-SPACING/2,-(SPACING/2),0]){
	cube([8*SPACING,HOLLOW_WALL,0.5*INCHES_TO_MM]);
}

